<?php

namespace Internetgalerie\IgDatapoolFe\Services\Exporter;

/**
 * Streamline Filter, that replaces \n with \r\n
 * This is used here, because we use an outputstream to php://output using fputcsv().
 */
class crlf_filter extends \php_user_filter
{
    public function filter($in, $out, &$consumed, $closing)
    {
        while ($bucket = stream_bucket_make_writeable($in)) {
            // make sure the line endings aren't already CRLF
            $bucket -> data = preg_replace("/(?<!\r)\n/", "\r\n", $bucket -> data);
            $consumed += $bucket -> datalen;
            stream_bucket_append($out, $bucket);
        }
        return PSFS_PASS_ON;
    }
}

/**
 * This class provides an export function for CSV files
 */
class CsvExporterService extends ExporterService
{
    protected $output;

    protected $options = array('filename' => '', 'delimiter' => ';', 'enclosure' => '""', 'lineending' => "\r\n", 'includeHeaders' => true);

    public function before()
    {
        header('Content-Type: text/csv; charset=utf-8');
        header('Content-Disposition: attachment; filename=' . $this -> options['filename'] . '.csv');
        $this -> output = fopen('php://output', 'w');
        if ($this -> options['lineending'] == "\r\n") {
            stream_filter_register('crlf', 'Internetgalerie\IgDatapoolFe\Services\Exporter\crlf_filter');
            stream_filter_append($this -> output, 'crlf', STREAM_FILTER_WRITE);
        }

        if ($this -> options['includeHeaders'] && count($this->getHeaders())) {
            $this -> exportLine($this -> getHeaders(), 0);
        }
    }

    public function after()
    {
        fclose($this -> output);
    }

    public function exportLine($array, $uid)
    {
        fputcsv($this -> output, $array, $this -> options['delimiter'], $this -> options['enclosure']);
    }
}
