<?php

namespace Internetgalerie\IgDatapoolFe\Validation\Validator;

/**
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

/**
 * Validator for DateTime objects.
 *
 * @api
 */
class DateTimeValidator extends \TYPO3\CMS\Extbase\Validation\Validator\DateTimeValidator
{
    use ValidatorTrait;

    protected $supportedOptions = ['compareWith' => ['', 'Date to compare with, as string d.m.Y', 'string'], 'compareWithField' => ['', 'Fieldname of field to compare the date with.', 'string'], 'operation' => ['', 'Operation: >,<,>=,<=,==', 'string']];

    /**
     * Checks if the given value is a valid DateTime object.
     *
     * If at least one error occurred, the result is FALSE.
     *
     * @param mixed $value The value that should be validated
     */
    public function isValid($value)
    {
        if (is_null($value)) {
            return;
        }

        if (!is_a($value, 'DateTime')) {
            $this -> addError($this -> translateErrorMessage('validator.datetime.notvalid', 'extbase', [gettype($value)]), 1238087674, [gettype($value)]);
            return;
        }

        if ($this -> options['compareWith'] || $this -> options['compareWithField']) {
            if ($this -> options['compareWith']) {
                $otherValue = \DateTime::createFromFormat('d.m.Y', $this -> options['compareWith']);
                if (!$otherValue) {
                    throw new \Exception('DateTimeValdiator, compareWith: Falsches Datumsformat');
                }
            }
            if ($this -> options['compareWithField'] && ($gov = GenericObjectValidator::$currentObjectValidator) != null) {
                $otherValue = $gov -> getCurrentObjectPropertyValue($this -> options['compareWithField']);
            }

            if (!is_a($otherValue, 'DateTime')) {
                return;
            }

            switch ($this->options['operation']) {
            case '<':
                if ($value >= $otherValue) {
                    $this -> addError($this -> translateErrorMessage('validator.datetime.notvalid', 'ig_render'), 1238087674);
                }
                break;
            case '>':
                if ($value <= $otherValue) {
                    $this -> addError($this -> translateErrorMessage('validator.datetime.notvalid', 'ig_render'), 1238087674);
                }
                break;
            case '<=':
                if ($value > $otherValue) {
                    $this -> addError($this -> translateErrorMessage('validator.datetime.notvalid', 'ig_render'), 1238087674);
                }
                break;
            case '>=':
                if ($value < $otherValue) {
                    $this -> addError($this -> translateErrorMessage('validator.datetime.notvalid', 'ig_render'), 1238087674);
                }
                break;
            case '=':
            case '==':
            default:
                if ($value != $otherValue) {
                    $this -> addError($this -> translateErrorMessage('validator.datetime.notvalid', 'ig_render'), 1238087674);
                }
            }
        }
    }
}
