<?php

namespace Internetgalerie\IgDatapoolFe\Validation\Validator;

class GenericObjectValidator extends \TYPO3\CMS\Extbase\Validation\Validator\GenericObjectValidator
{
    public static $currentField = null;
    public static $currentObjectValidator = null;
    public static $currentObject = null;

    /**
     * Checks if the specified property of the given object is valid, and adds
     * found errors to the $messages object.
     *
     * @param  mixed  $value        The value to be validated
     * @param  array  $validators   The validators to be called on the value
     * @param  string $propertyName Name of ther property to check
     * @return void
     */
    protected function checkProperty($value, $validators, $propertyName)
    {
        self::$currentField = $propertyName;
        self::$currentObjectValidator = $this;
        $result = null;
        foreach ($validators as $validator) {
            if ($validator instanceof ObjectValidatorInterface) {
                $validator -> setValidatedInstancesContainer($this -> validatedInstancesContainer);
            }

            $currentResult = $validator -> validate($value);
            if ($currentResult -> hasMessages()) {
                if ($result == null) {
                    $result = $currentResult;
                } else {
                    $result -> merge($currentResult);
                }
            }
        }
        if ($result != null) {
            $this -> result -> forProperty($propertyName) -> merge($result);
        }
        self::$currentField = null;
        self::$currentObjectValidator = null;
    }

    /**
     * Checks if the given value is valid according to the property validators.
     * Only checks DIRTY property, used for tabsAsSteps
     *
     * @param  mixed $object The value that should be validated
     * @return void
     * @api
     * @todo:  method must be protected once the old property mapper is removed
     */
    public function isValid($object)
    {
        self::$currentObject = $object;

        $objectArguments = false;
        if (\Internetgalerie\IgDatapoolFe\Controller\ActionController::$currentController != null
            && \Internetgalerie\IgDatapoolFe\Controller\ActionController::$currentController -> getActionMethodName() == 'stepAction'
        ) {
            $objectArguments = array_keys(\Internetgalerie\IgDatapoolFe\Controller\ActionController::$currentController -> getObjectArguments());
        }

        foreach ($this->propertyValidators as $propertyName => $validators) {
            // check only submitted aprams
            if (is_array($objectArguments) && !in_array($propertyName, $objectArguments)) {
                continue;
            }


            $propertyValue = $this -> getPropertyValue($object, $propertyName);
            $this -> checkProperty($propertyValue, $validators, $propertyName);
        }
        self::$currentObject = null;
    }

    public function getCurrentObjectPropertyValue($propertyName)
    {
        if (self::$currentObject == null) {
            return '';
        }
        return $this -> getPropertyValue($this -> currentObject, $propertyName);
    }
}
