<?php

namespace Internetgalerie\IgDatapoolFe\Validation\Validator;

/**
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

/**
 * Validator for not empty values.
 *
 * @api
 */
class NotEmptyValidator extends \TYPO3\CMS\Extbase\Validation\Validator\NotEmptyValidator
{
    use ValidatorTrait;

    /**
     * @var array
     */
    protected $supportedOptions = ['if' => ['', 'Dependant Field', 'string'], 'value' => ['', 'Dependant Fields value', 'string']];

    /**
     * Checks if the given property ($propertyValue) is not empty (NULL, empty string, empty array or empty object).
     *
     * If at least one error occurred, the result is FALSE.
     *
     * @param  mixed $value The value that should be validated
     * @return void
     */
    public function isValid($value)
    {
        if ($this -> options['if']) {
            //check other field
            if (($gov = GenericObjectValidator::$currentObjectValidator) != null) {
                $otherValue = $gov -> getCurrentObjectPropertyValue($this -> options['if']);

                //check if othervaleu is set to the given value, then
                if ($this->options['value'] && $otherValue != $this->options['value']) {
                    return;
                } elseif ($this -> _isEmpty($otherValue)) {
                    return;
                }
            }
        }

        // if this is empty, error!
        if ($this -> _isEmpty($value)) {
            $this -> addError($this -> translateErrorMessage('validator.notempty.empty', 'ig_render'), 1221560910);
        }
    }

    /**
     * Check if the value is empty / not set / null / or no options given
     *
     * @param  mixed $value value to check
     * @return bool
     */
    protected function _isEmpty($value)
    {
        if ($value === null) {
            return true;
        }
        if ($value === '') {
            return true;
        }
        if (is_array($value) && empty($value)) {
            return true;
        }
        if (is_object($value) && $value instanceof \Countable && $value -> count() === 0) {
            return true;
        }
        return false;
    }
}
