<?php

namespace Internetgalerie\IgDatapoolFe\Validation\Validator;

/**
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

/**
 * Validator for Urls
 *
 * @api
 */
use Internetgalerie\IgDatapoolFe\Validation\Validator;

class UniqueValidator extends \TYPO3\CMS\Extbase\Validation\Validator\AbstractValidator
{
    use ValidatorTrait;

    protected $supportedOptions = ['maxItems' => ['1', 'Maximum Items allowed', 'string']];

    /**
     * Checks if the given value is unique througout all records in the db
     *
     * @param  mixed $value The value that should be validated
     * @return void
     * @api
     */
    public function isValid($value)
    {
        //the value to campare the count with, default: 1
        $compare = intval($this -> options['maxItems']);

        //the id that was currently submitted might be empty!!
        $submittedUid = \TYPO3\CMS\Extbase\Reflection\ObjectAccess::getProperty(GenericObjectValidator::$currentObject, 'uid');

        $method = 'findBy' . ucfirst(GenericObjectValidator::$currentField);
        $list = \Internetgalerie\IgDatapoolFe\Controller\ActionController::$currentRepository -> {$method}($value);
        $count = count($list);

        /**
         * If we EDIT a record, then we search for the current record in the results.
         * if found, we don't count the current records, cause it is the one we edit.
         */
        if (\TYPO3\CMS\Core\Utility\MathUtility::canBeInterpretedAsInteger($submittedUid)) {
            foreach ($list as $elem) {
                $uid = \TYPO3\CMS\Extbase\Reflection\ObjectAccess::getProperty($elem, 'uid');
                /*
                 * in case the current UID exists in the database, i.e. we update it, then we reduce the comparing value by 1,
                 * because it is the record itself. does this make sense?
                 * THE ELEMENT ITSELF DOESNT ADD TO COUNT!
                 */
                if ($uid == $submittedUid) {
                    --$count;
                }
            }
        }

        /**
         * If there are more records than allowed, add an error
         */
        if ($count >= $compare) {
            $this -> addError($this -> translateErrorMessage('validator.unique.notunique', 'ig_render'), 1221559976);
        }
    }
}
